# Generated by Django 2.2.11 on 2020-03-20 09:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_yubin', '0007_auto_20200319_1158'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='log',
            name='result',
        ),
        migrations.DeleteModel(
            name='QueuedMessage',
        ),
    ]
