# Generated by Django 3.2.16 on 2022-11-22 16:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_yubin', '0008_auto_20200320_0407'),
    ]

    operations = [
        migrations.AlterField(
            model_name='log',
            name='date',
            field=models.DateTimeField(auto_now_add=True, verbose_name='date'),
        ),
        migrations.RenameField(
            model_name='message',
            old_name='encoded_message',
            new_name='_message_data',
        ),
        migrations.AlterField(
            model_name='message',
            name='_message_data',
            field=models.TextField(db_column='message_data', verbose_name='message data'),
        ),
    ]
